<?php
/**
 * This file is part of OpenMediaVault-autoshutdown.
 *
 * @license   http://www.gnu.org/licenses/gpl.html GPL Version 3
 * @author    Marcel Beck <marcel.beck@mbeck.org>
 * @copyright Copyright (c) 2012 Marcel Beck
 *
 * OpenMediaVault is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * OpenMediaVault is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenMediaVault. If not, see <http://www.gnu.org/licenses/>.
 */
require_once("openmediavault/module.inc");
require_once("openmediavault/config.inc");
require_once("openmediavault/error.inc");
require_once("openmediavault/util.inc");
require_once("openmediavault/initscript.inc");
require_once("openmediavault/log.inc");

/**
 * Autoshutdown Module
 */
class AutoshutdownModule extends OMVModuleAbstract implements OMVINotifyListener, OMVIModuleStatus
{
    /**
     * Get the module name.
     *
     * @return string
     */
    public static function getName()
    {
        return "autoshutdown";
    }

    /**
     * Get the module status.
     *
     * @throws OMVException
     * @return array
     */
    public function getStatus()
    {
        global $xmlConfig;
        // Get the configuration object
        $xpath  = "//services/autoshutdown";
        $object = $xmlConfig->get($xpath);
        if (is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }
        // Check if process is running. Init script does not support 'status'.
        $cmd = "ps --no-heading -C autoshutdown.sh";
        OMVUtil::exec($cmd, $output, $result);
        return array(
            "name"    => $this->getName(),
            "title"   => gettext("Autoshutdown"),
            "enabled" => boolval($object['enable']),
            "running" => !empty($output[0])
        );
    }

    /**
     * Update the configuration.
     */
    public function mkConf()
    {
        $cmd = "sudo omv-mkconf autoshutdown 2>&1";
        OMVUtil::exec($cmd, $output, $result);
        if ($result !== 0) {
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED, $cmd, implode("\n", $output));
        }
    }

    /**
     * Update the service.
     */
    public function updateService()
    {
        global $xmlConfig;
        // Get the configuration object
        $xpath  = "//services/autoshutdown";
        $object = $xmlConfig->get($xpath);
        if (is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }
        // Create configuration
        $this->mkConf();

        switch (boolval($object['enable'])) {
            case true:
                $initd = new OMVSysVInitScript("autoshutdown", true);
                $initd->exec();
                break;
            default:
                $initd = new OMVSysVInitScript("autoshutdown", false);
                $initd->exec();
                break;
        }
    }

    /**
     * Bind listeners.
     *
     * @param \OMVNotifyDispatcher $dispatcher
     */
    function bindListeners(OMVNotifyDispatcher $dispatcher)
    {
        // Update service if configuration has been modified
        $dispatcher->addListener(OMV_NOTIFY_MODIFY, "org.openmediavault.services.autoshutdown", array($this, "updateService"), 1);
        // Update Zeroconfig if configuration has been modified
        $moduleMgr = &OMVModuleMgr::getInstance();
        $dispatcher->addListener(OMV_NOTIFY_MODIFY, "org.openmediavault.services.autoshutdown", array($moduleMgr->getModule("zeroconf"), "mkConf"));
    }
}

// Register module
$moduleMgr = &OMVModuleMgr::getInstance();
$moduleMgr->registerModule(new AutoshutdownModule());

// Register new logfile type
OMVLogFile::registerType(
    "autoshutdown",
    array(
         "filename" => "autoshutdown.log",
         "filepath" => "/var/log/autoshutdown.log",
         "regex"    => "/^(\S+\s+\d+\s+\S+)\s+.*\s+logger:\s+\S+:\s+(\S+):\s+(.*)$/",
         "columns"  => array(
             "date" => array(
                 "index" => 1,
                 "func"  => create_function('$v', 'return strpdate($v,"M j G:i:s");')
             ),
             "type" => 2,
             "log"  => 3
         )
    )
);
