<?php
/**
 * This file is part of OpenMediaVault-autoshutdown.
 *
 * @license   http://www.gnu.org/licenses/gpl.html GPL Version 3
 * @author    Marcel Beck <marcel.beck@mbeck.org>
 * @copyright Copyright (c) 2012 Marcel Beck
 *
 * OpenMediaVault is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * OpenMediaVault is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with OpenMediaVault. If not, see <http://www.gnu.org/licenses/>.
 */
require_once("openmediavault/object.inc");
require_once("openmediavault/error.inc");
require_once("openmediavault/util.inc");
require_once("openmediavault/rpc.inc");
require_once("openmediavault/notify.inc");

/**
 * Autoshutdown RPC Handler
 */
class AutoshutdownRpc extends OMVRpc
{

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->methodSchemata = array(
            "setSettings"   => array(
                '{
					"type":"object",
					"properties":{
						"enable":{"type":"boolean"},
						"flag":{"type":"integer"},
						"sleep":{"type":"integer"},
						"checkclockactive":{"type":"boolean"},
						"uphours-begin":{"type":"integer"},
						"uphours-end":{"type":"integer"},
						"syslog":{"type":"boolean"},
						"verbose":{"type":"boolean"},
						"fake":{"type":"boolean"},
						"extraoptions":{"type":"string"},
						"autoshutdown_version":{"type":"string","optional":true}
					}
				}'
            ),
            "setNetwork"    => array(
                '{
					"type":"object",
					"properties":{
						"range":{"type":"string"},
						"custom_ports":{"type":"string"},
						"nw_intensesearch":{"type":"boolean"}
					}
				}'
            ),
            "setPorts"      => array(
                '{
					"type":"object",
					"properties":{
						"ports":{
							"type":"array",
							"items":{
								"type":"object",
								"properties":{
									"protocol":{"type":"string"},
									"port":{"type":"integer"},
									"check":{"type":"boolean"}
								}
							}
						}
					}
				}'
            )
        );
    }

    /**
     * Get configuration object.
     */
    function getSettings()
    {
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Get configuration object
        $xpath  = "//services/autoshutdown";
        $object = $xmlConfig->get($xpath);
        if (is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }
        // Modify result data
        $object['flag']             = intval($object['flag']);
        $object['sleep']            = intval($object['sleep']);
        $object['uphours-begin']    = intval($object['uphours-begin']);
        $object['uphours-end']      = intval($object['uphours-end']);
        $object['checkclockactive'] = boolval($object['checkclockactive']);
        $object['syslog']           = boolval($object['syslog']);
        $object['verbose']          = boolval($object['verbose']);
        $object['fake']             = boolval($object['fake']);

        $cmd = "sudo dpkg -p autoshutdown | grep 'Version:' | sed -r 's/Version: //g'";
        OMVUtil::exec($cmd, $output, $result);
        if ($result !== 0) {
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED, $cmd, implode("\n", $output));
        }
        if (isset($output[0])) {
            $object['autoshutdown_version'] = $output[0];
        }

        return $object;
    }

    /**
     * Set configuration object.
     */
    function setSettings($data)
    {
        //		$this->debug(var_export(func_get_args(), TRUE));
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Validate method parameters
        $this->validateParams(__METHOD__, func_get_args());

        // Get existing configuration object
        $xpath             = "//services/autoshutdown";
        $old_configuration = $xmlConfig->get($xpath);
        if (is_null($old_configuration)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }

        // Prepare configuration data
        $object = array(
            "enable"              => array_boolval($data, 'enable'),
            "flag"                => intval($data['flag']),
            "sleep"               => intval($data['sleep']),
            "checkclockactive"    => array_boolval($data, 'checkclockactive'),
            "uphours-begin"       => intval($data['uphours-begin']),
            "uphours-end"         => intval($data['uphours-end']),
            "syslog"              => array_boolval($data, 'syslog'),
            "verbose"             => array_boolval($data, 'verbose'),
            "fake"                => array_boolval($data, 'fake'),
            "extraoptions"        => $data['extraoptions'],
            "network"             => isset($old_configuration['network']) ? $old_configuration['network'] : array(),
        );
        // Set configuration object
        if (false === $xmlConfig->replace($xpath, $object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);
        }
        // Save configuration
        if (false === $xmlConfig->save()) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SAVE_FAILED, $xmlConfig->getError());
        }
        // Notify configuration changes
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(OMV_NOTIFY_MODIFY, "org.openmediavault.services.autoshutdown");
    }

    /**
     * Get configuration object.
     */
    function getNetwork()
    {
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Get configuration object
        $xpath  = "//services/autoshutdown/network";
        $object = $xmlConfig->get($xpath);
        if (is_null($object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }

        $object['nw_intensesearch'] = boolval($object['nw_intensesearch']);

        return $object;
    }

    /**
     * Set configuration object.
     */
    function setNetwork($data)
    {
        //		$this->debug(var_export(func_get_args(), TRUE));
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Validate method parameters
        $this->validateParams(__METHOD__, func_get_args());

        // Get existing configuration object
        $xpath             = "//services/autoshutdown/network";
        $old_configuration = $xmlConfig->get($xpath);
        if (is_null($old_configuration)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }

        // Prepare configuration data
        $object = array(
            'range'           => $data['range'],
            'custom_ports'    => $data['custom_ports'],
            "nw_intensesearch"=> array_boolval($data, 'nw_intensesearch'),
            'ports'           => isset($old_configuration['ports']) ? $old_configuration['ports'] : array(),
        );
        // Set configuration object
        if (false === $xmlConfig->replace($xpath, $object)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);
        }
        // Save configuration
        if (false === $xmlConfig->save()) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SAVE_FAILED, $xmlConfig->getError());
        }
        // Notify configuration changes
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(OMV_NOTIFY_MODIFY, "org.openmediavault.services.autoshutdown");
    }

    /**
     * Get list of used ports
     *
     * @return array
     * @throws OMVException
     */
    function getPorts()
    {
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Get configuration object
        $xpath        = "//services/autoshutdown/network/ports/port";
        $config_ports = $xmlConfig->getList($xpath);
        if (is_null($config_ports)) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_GET_OBJECT_FAILED, $xpath);
        }

        $cmd = "sudo netstat -tulpen | egrep -v 'monit|rpc|master|portmap| -' | egrep '^(tcp\\ |udp\\ )' | sed -r 's/LISTEN//g; s/([0-9]+.){3}[0-9]+://g; s/[0-9]+\///g; s/://g' | awk '{print $1 \";\" $4 \";\" $8}' | sort -u";
        OMVUtil::exec($cmd, $output, $result);
        if ($result !== 0) {
            throw new OMVException(OMVErrorMsg::E_EXEC_FAILED, $cmd, implode("\n", $output));
        }
        $return = array();
        foreach ($output as $key => $value) {
            $tmp  = explode(';', $value);
            $port = array(
                'protocol' => $tmp[0],
                'port'     => intval($tmp[1]),
                'program'  => $tmp[2],
                'check'    => false,
            );

            foreach ($config_ports as $config_port) {
                if (($config_port['port'] == $port['port']) and ($config_port['protocol'] == $port['protocol'])) {
                    $port['check'] = array_boolval($config_port, 'check');
                }
            }

            $return[] = $port;
        }
        return $return;
    }

    /**
     * Set ports
     *
     * @param $data
     *
     * @throws OMVException
     */
    public function setPorts($data)
    {
        global $xmlConfig;
        // Check permissions
        $this->validateSession();
        if (!$this->hasRole(OMV_ROLE_ADMINISTRATOR)) {
            throw new OMVException(OMVErrorMsg::E_RPC_SERVICE_INVALID_PERMISSION);
        }
        $this->commitSession();
        // Validate method parameters
        $this->validateParams(__METHOD__, func_get_args());

        $xmlConfig->delete('//services/autoshutdown/network/ports/port');

        $success = true;
        foreach ($data['ports'] as $value) {
            $success = $xmlConfig->set("//services/autoshutdown/network/ports", array("port" => $value));
        }
        if (false === $success) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SET_OBJECT_FAILED);
        }
        // Save configuration
        if (false === $xmlConfig->save()) {
            throw new OMVException(OMVErrorMsg::E_CONFIG_SAVE_FAILED, $xmlConfig->getError());
        }
        // Notify configuration changes
        $dispatcher = &OMVNotifyDispatcher::getInstance();
        $dispatcher->notify(OMV_NOTIFY_MODIFY, "org.openmediavault.services.autoshutdown");
    }
}
